# PROGRAMME TERMINALE SPECIALITE 2021
# -----------------------------------
# Illustration de la seconde loi de Kepler
#-----------------------------------------------------------------------
# Code proposé par : Guillaume Dewaele (professeur en CPGE, Anger)
# guillaume.dewaele@sci-phy.org
#-----------------------------------------------------------------------
# légères modifications: O. Chaumette (pause, commentaires, remise en forme du code)
# avec l'autorisation de l'auteur
#-----------------------------------------------------------------------

""" Paramètres modifiables par les élèves """

# Excentricité
e = 0.8

# Nombre de corps sur l'orbite
Nc = 4

# Taille du secteur
Ns = 100

# Vitesse
speed = 10


""" FIN de la partie  modifiable par les élèves """

#-----------------------------------------------------------------------
# ############################
# Bibliothèques utilisées
# ############################
import numpy as np
import math
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.animation as ani
import matplotlib.patches as ptc
from scipy.integrate import odeint
import itertools as itt


# ############################
# FONCTIONS
# ############################

# fonction mettant en pause ou non l'animation si on clique sur la fenêtre matplotlib
def Clic(event):
    global animation_en_pause
    if not animation_en_pause:
        anim.event_source.stop()
        animation_en_pause = True
    else:
        anim.event_source.start()
        animation_en_pause = False


# fonction qui calcule le coefficient entre tan (1/2 angle de P/au foyer) et tan(1/2 angle de P sur cercle (anomalie excentrique))
def Foo(e) :
    return (1+e)/(1-e**2)**0.5


# Calcul des dérivées
def Der(Y, t) :
    x, y, z, vx, vy, vz = Y
    ax = -G * MS / (x**2+y**2+z**2)**1.5 * x
    ay = -G * MS / (x**2+y**2+z**2)**1.5 * y
    az = -G * MS / (x**2+y**2+z**2)**1.5 * z
    return [vx, vy, vz, ax, ay, az]


# calcul des coordonnées
def GetCoords(i, mask=False) :
    T = np.array([[0]+list(res[i%1260:i%1260+Ns,0]), [0]+list(res[i%1260:i%1260+Ns,1])]).T
    if mask :
        T *= 0
    return T

# fonctions gérant l'animation
# ----------------------------
# gestion de la mise à l'échelle
def SizeChanged(ax, old=[]) :
    current = [ ax.bbox.width, ax.bbox.height ]
    if old != current :
        old[:] = current
        return True
    return False


# fonction initialisation de l'animation
def Init() :
    for k, poly, corps in polys :
        crds = GetCoords(k+i, True)
        poly.set_xy(crds)
        corps.set_data(np.ma.array([crds[-1,0]], mask=True),
                       np.ma.array([crds[-1,1]], mask=True))
    return [ poly for k, poly, _ in polys ] + [ corps for k, _, corps in polys ]


# fonction appelée à chaque frame
def Update(i) :
    for k, poly, corps in polys :
        crds = GetCoords(k+i)
        poly.set_xy(crds)
        corps.set_data([crds[-1,0]], [crds[-1,1]])
    if SizeChanged(plt.gca()) :
        plt.gcf().canvas.draw()
    return [ poly for k, poly, _ in polys ] + [ corps for k, _, corps in polys ]

#----------------------------------------------------------------------------------


# ############################
# Initialisation des variables/constantes
# ############################

# Quelques constantes
au = 149597870700.0             # ua en km
G = 6.6742e-11                  # SI
MS = 1.989e30                   # kg masse Soleil

# variable booléenne permettant de savoir si l'animation est en pause ou non
animation_en_pause = False

# condition initiale
C0 = [ (1+e)*au, 0, 0, 0, 6.283*au/(365.25*86400)/Foo(e), 0 ]


T = np.linspace(0.0, 86400*365*2, 1260*2)       # tableau de dates
res = odeint(Der, C0, T)            # resolution equation de Kepler

#-----------------------------------------------------------------------
# ############################
# Code principal
# ############################

# Détection utilisation hors Pyzo
if '__iep__' not in globals() :
    matplotlib.interactive(False)

# ############################
# Figure Matplotlib
# ############################
# initialisation de la figure:
Figure = plt.figure("2nde loi de Kepler")
# Création d'un évènement "Clic de la souris" sur la fenêtre Matplolib (si clic, appel de la fonction "Clic")
Figure.canvas.mpl_connect('button_press_event', Clic)
# tracé de l'origine (soleil) et de l'ellipse de l'orbite
plt.plot(0,0,"w.",label="\"clic\" = PAUSE")
plt.plot([0], [0], "yo", markersize = 20)
plt.plot(res[:1260,0], res[:1260,1], 'k--')

plt.legend(loc='upper right')
# gestion des axes/titres
plt.axis("equal")                   # orthonormé
plt.title("Deuxième loi de Kepler")
# on enlève les graduations des axes (plus joli !)
axes=plt.gca()
axes.set_xticklabels([])
axes.set_xticks([])
axes.set_yticklabels([])
axes.set_yticks([])


# Construit la liste des polygones et corps célestes
polys = []
for i in range(Nc) :
    polys.append((i*1260//Nc, ptc.Polygon(GetCoords(0, True), True, color='ygbrmc'[i%6], alpha=0.3), plt.plot([0],[0],'ygbrmc'[i%6]+'o', markersize=8)[0]))
    plt.gca().add_patch(polys[-1][1])

# animation de la figure
anim = ani.FuncAnimation(plt.gcf(), Update, frames=itt.count(0, int(speed)),
                         interval=50, blit=True, init_func=Init)


# Détection utilisation hors Pyzo
if '__iep__' not in globals() :
    plt.show()
